GS = 'Gold standard'
SUPERKINGDOM = 'superkingdom'
PHYLUM = 'phylum'
CLASS = 'class'
ORDER = 'order'
FAMILY = 'family'
GENUS = 'genus'
SPECIES = 'species'
STRAIN = 'strain'
PHYLUM_SPECIES = [PHYLUM, CLASS, ORDER, FAMILY, GENUS, SPECIES]
ALL_RANKS = [SUPERKINGDOM, PHYLUM, CLASS, ORDER, FAMILY, GENUS, SPECIES, STRAIN]

UNIFRAC = 'Weighted UniFrac error'
UNW_UNIFRAC = 'Unweighted UniFrac error'
L1NORM = 'L1 norm error'
PRECISION = 'Purity (precision)'
RECALL = 'Completeness (recall)'
F1_SCORE = 'F1 score'
TP = "True positives"
FP = 'False positives'
FN = "False negatives"
OTUS = "Taxon counts"
JACCARD = "Jaccard index"
SHANNON_DIVERSITY = 'Shannon diversity'
SHANNON_EQUIT = 'Shannon equitability'
BRAY_CURTIS = 'Bray-Curtis distance'
ALL_METRICS = [UNIFRAC, UNW_UNIFRAC, L1NORM, PRECISION, RECALL, F1_SCORE, TP, FP, FN, JACCARD, SHANNON_DIVERSITY, SHANNON_EQUIT, BRAY_CURTIS]

TOOLTIP_UNIFRAC = 'tree-based measure of similarity between the true and predicted abundances at all taxonomic ranks ranging from 0 (high similarity) to 16 (low similarity).'
TOOLTIP_UNW_UNIFRAC = 'similar to the weighted UniFrac error, but measuring how well a profiler correctly identified the presence and absence of taxa instead of their relative abundances. The maximal value is the product of the number of taxonomic ranks minus 1 and the number of true taxa at all taxonomic ranks.'
TOOLTIP_L1NORM = 'sum of the absolute differences between the true and predicted abundances at the selected taxonomic rank ranging from 0 (perfect reconstruction of the relative abundances) to 2 (totally incorrect reconstruction).'
TOOLTIP_PRECISION = 'ratio of taxa correctly predicted as present and all predicted taxa at the selected taxonomic rank. It ranges from 0 (worst) to 1 (best).'
TOOLTIP_RECALL = 'ratio of taxa correctly predicted as present and all taxa present at the selected taxonomic rank. It ranges from 0 (worst) to 1 (best).'
TOOLTIP_F1_SCORE = 'harmonic average of completeness and purity at the selected taxonomic rank. It ranges from 0 (worst) to 1 (best).'
TOOLTIP_TP = "number of taxa correctly predicted as present at the selected taxonomic rank."
TOOLTIP_FP = 'number of taxa incorrectly predicted as present at the selected taxonomic rank.'
TOOLTIP_FN = "number of taxa incorrectly predicted as absent at the selected taxonomic rank."
TOOLTIP_OTUS = "number of predicted taxa at the selected taxonomic rank."
TOOLTIP_JACCARD = "indicator of similarity between the sets of true and predicted taxa at the seletec taxonomic rank computed as the ratio of the number of taxa in the intersection of these sets to the number of taxa in their union. It ranges from 0 (complete dissimilarity) to 1 (complete overlap)."
TOOLTIP_SHANNON_DIVERSITY = 'ranges from 0 to the natural logarithm of the number of predicted taxa, where the latter represents maximal possible diversity with all taxa being evenly represented.'
TOOLTIP_SHANNON_EQUIT = 'normalized form of Shannon diversity index ranging from 0 to 1 with 1 indicating complete evenness.'
TOOLTIP_BRAY_CURTIS = 'normalized form of the L1 norm ranging from 0 to 1.'
